/* This file (carrier.c) does PWM on the tn85 T1 timer to generate a 250 kHz carrier wave that could be used for 8-bit PCM.
 * Modulation is demonstrated in modulate.c and the two will later be merged into a tn85 PCM library.
 * This is a modified version of t85-pwm-demo.c.
 *
 * Copyright 2022 Johan Carlsson
 *
 */

#include <avr/io.h>
#define F_CPU 1000000
#include <util/delay.h>

int main()
{
	/* change clock source from CK to PCK */
	/* the three statements below MUST BE done one by one and in that order */
	PLLCSR |= (1 << PLOCK);
	do _delay_us(100.); while (PLLCSR & PLOCK); /* wait 100 us and check that PLOCK has been set */
	PLLCSR |= (1 << PLLE);
	PLLCSR |= (1 << PCKE);
	/* PCK is now the clock source for T1, and it's running at 64 MHz */

	/*** SET UP TIMER1 TO DO PWM ON PB1 ***/

	/* configure prescaler (CS1[0:3] = 0001 => 1, CS1[0:3] = 0010 => 1/2, etc.) */
	TCCR1 |= (1 << CS10);
	TCCR1 &= ~((1 << CS11) | (1 << CS12) | (1 << CS13));
	GTCCR |= (1 << PSR1); /* effect the new PS setting */

	OCR1C = 255;
	/* set duty cycle (OCR1A + 1) / (OCR1C + 1) for T1 comparator A */
	OCR1A = 127;

	/* put PWM output signal on PB1, nothing on PB0 (see Table 12-1 in datasheet) */
	TCCR1 &= ~(1 << COM1A0);
	TCCR1 |= (1 << COM1A1);

	/* set PWM mode for comparator OCR1A */
	TCCR1 |= (1 << PWM1A);

	/* non-timer initialization */
	DDRB |= (1 << DDB1); /* make PB1 (a.k.a. OC1A) an output pin */

	/* doesn't look like it's needed for intrinsic interrupts */
	/* sei(); */ /* enable interrupts */

	while (1);
}
