/* This file (modulate.c) uses Comparator A on the tn85 T0 timer to toggle an LED once per second.
 * PCM modulation would be done in the same fashion, but with 8 kHz interrupt frequency, and an
 * audio sample copied from flash and written to a T1 register instead of an LED being toggled.
 * Generation of a PCM carrier wave is demonstrated in carrier.c and the two will later be merged 
 * into a tn85 PCM library.
 *
 * Copyright 2022 Johan Carlsson
 *
 */

#include <avr/interrupt.h>
#include <avr/io.h>

unsigned char nint;

/* gets called at around 10 Hz */
ISR(TIM0_COMPA_vect)
{
	/* toggle PB4 about once per second */
	nint = nint % 10;
	if (9 == nint)
		PINB |= (1 << PINB4);
	nint++;
}

int main()
{
	/*** SET UP TIMER0 ***/

	/* prescale T0 to run at 976.5625 Hz (1 MHz / 1,024) */
	TCCR0B = (TCCR0B & ~(1 << CS01)) | ((1 << CS00) | (1 << CS02));

	/* set CTC mode for T0 (see Table 11-5 in datasheet) */
	TCCR0A &= ~(1 << WGM00);
	TCCR0A |= (1 << WGM01);
	TCCR0B &= ~(1 << WGM02);

	/* set the compare register */
	OCR0A = 97; /* 976.5625 Hz / (97 + 1) = 9.965 Hz */

	/* enable interrupts */
	sei();

	/* fire interrupt when TCNT0 == OCR0A */
	TIMSK |= (1 << OCIE0A);

	/* make PB4 an output pin */
	DDRB |= (1 << DDB4);
	/* drive PB4 high */
	PORTB = (PORTB4);

	/* initialize global counter (used in ISR) */
	nint = 0;

	while (1);
}
